import { BREAKPOINT_KEYS } from "./muiBreakpoints";

/**
 *
 *
 * @param getValue the value the result object (undefined will be remove from result)
 * @param options
 *  - breakpoints: list of Breakpoints to be iterate
 *  - initialValue: if provided, will be used to compare if the value from getValue should be added or not
 * @returns object that can be passed to <MuiComponent sx={...} />
 */
export var createSxResult = (getValue, options) => {
  var {
    breakpoints = BREAKPOINT_KEYS,
    initialValue
  } = options !== null && options !== void 0 ? options : {};
  var result = {};
  var lastValue = undefined;

  for (var _bp of breakpoints) {
    var value = getValue(_bp, lastValue);

    if (value !== undefined) {
      // only assign value if not undefined
      if (lastValue === undefined) {
        // 1st valid value
        result[_bp] = value;
        lastValue = value;

        if (initialValue !== undefined && initialValue === value) {
          // if same as initial, deleted from object
          delete result[_bp];
        }
      } else {
        if (lastValue !== value) {
          // next valid value that is not the same as lastValue and so on
          result[_bp] = value;
          lastValue = value;
        }
      }
    }
  }

  return result;
};